@@ -70,7 +70,7 @@ class WaitResult(
     __slots__ = ()
 
 
-class Node(_common.ListMixin, resource.Resource):
+class Node(_common.Resource):
     resources_key = 'nodes'
     base_path = '/nodes'
 
@@ -115,7 +115,7 @@ class Node(_common.ListMixin, resource.Resource):
     chassis_id = resource.Body("chassis_uuid")
     #: The current clean step.
     clean_step = resource.Body("clean_step")
-    #: Hostname of the conductor currently handling this ndoe. Added in API
+    #: Hostname of the conductor currently handling this node. Added in API
     # microversion 1.49.
     conductor = resource.Body("conductor")
     #: Conductor group this node is managed by. Added in API microversion 1.46.
@@ -124,6 +124,8 @@ class Node(_common.ListMixin, resource.Resource):
     created_at = resource.Body("created_at")
     #: The current deploy step. Added in API microversion 1.44.
     deploy_step = resource.Body("deploy_step")
+    #: The description of the node. Added in API microversion 1.51.
+    description = resource.Body("description")
     #: The name of the driver.
     driver = resource.Body("driver")
     #: All the metadata required by the driver to manage this node. List of
@@ -164,6 +166,9 @@ class Node(_common.ListMixin, resource.Resource):
     #: Any error from the most recent transaction that started but failed to
     #: finish.
     last_error = resource.Body("last_error")
+    #: Field indicating if the node is leased to a specific project.
+    #: Added in API version 1.65
+    lessee = resource.Body("lessee")
     #: A list of relative links, including self and bookmark links.
     links = resource.Body("links", type=list)
     #: user settable description of the reason why the node was placed into
@@ -270,7 +275,7 @@ class Node(_common.ListMixin, resource.Resource):
             # API version 1.1 uses None instead of "available". Make it
             # consistent.
             attrs['provision_state'] = 'available'
-        return super(Node, self)._consume_body_attrs(attrs)
+        return super()._consume_body_attrs(attrs)
 
     def create(self, session, *args, **kwargs):
         """Create a remote resource based on this instance.
@@ -341,9 +346,7 @@ class Node(_common.ListMixin, resource.Resource):
         # Ironic cannot set provision_state itself, so marking it as unchanged
         self._clean_body_attrs({'provision_state'})
 
-        super(Node, self).create(
-            session, *args, microversion=microversion, **kwargs
-        )
+        super().create(session, *args, microversion=microversion, **kwargs)
 
         if (
             expected_provision_state == 'manageable'
@@ -390,7 +393,7 @@ class Node(_common.ListMixin, resource.Resource):
                 # the new status.
                 return self.fetch(session)
 
-        return super(Node, self).commit(session, *args, **kwargs)
+        return super().commit(session, *args, **kwargs)
 
     def set_provision_state(
         self,
@@ -462,6 +465,14 @@ class Node(_common.ListMixin, resource.Resource):
                     'Config drive can only be provided with '
                     '"active" and "rebuild" targets'
                 )
+            if isinstance(config_drive, bytes):
+                try:
+                    config_drive = config_drive.decode('utf-8')
+                except UnicodeError:
+                    raise ValueError(
+                        'Config drive must be a dictionary or a base64 '
+                        'encoded string'
+                    )
             # Not a typo - ironic accepts "configdrive" (without underscore)
             body['configdrive'] = config_drive
 
@@ -711,17 +722,15 @@ class Node(_common.ListMixin, resource.Resource):
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'management', 'inject_nmi')
 
-        body = {}
-
         response = session.put(
             request.url,
-            json=body,
+            json={},
             headers=request.headers,
             microversion=version,
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to inject NMI to node {node}".format(node=self.id)
+        msg = f"Failed to inject NMI to node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def set_power_state(self, session, target, wait=False, timeout=None):
@@ -813,7 +822,7 @@ class Node(_common.ListMixin, resource.Resource):
         body = {'id': vif_id}
         retriable_status_codes = _common.RETRIABLE_STATUS_CODES
         if not retry_on_conflict:
-            retriable_status_codes = set(retriable_status_codes) - {409}
+            retriable_status_codes = list(set(retriable_status_codes) - {409})
         response = session.post(
             request.url,
             json=body,
@@ -931,13 +940,13 @@ class Node(_common.ListMixin, resource.Resource):
             request.url, headers=request.headers, microversion=version
         )
 
-        msg = "Failed to validate node {node}".format(node=self.id)
+        msg = f"Failed to validate node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
         result = response.json()
 
         if required:
             failed = [
-                '%s (%s)' % (key, value.get('reason', 'no reason'))
+                '{} ({})'.format(key, value.get('reason', 'no reason'))
                 for key, value in result.items()
                 if key in required and not value.get('result')
             ]
@@ -1041,7 +1050,7 @@ class Node(_common.ListMixin, resource.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to set boot device for node {node}".format(node=self.id)
+        msg = f"Failed to set boot device for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def get_supported_boot_devices(self, session):
@@ -1106,7 +1115,7 @@ class Node(_common.ListMixin, resource.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to change boot mode for node {node}".format(node=self.id)
+        msg = f"Failed to change boot mode for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def set_secure_boot(self, session, target):
@@ -1240,7 +1249,7 @@ class Node(_common.ListMixin, resource.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to set traits for node {node}".format(node=self.id)
+        msg = f"Failed to set traits for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
         self.traits = traits
@@ -1258,7 +1267,7 @@ class Node(_common.ListMixin, resource.Resource):
         version = self._get_microversion(session, action='commit')
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(
-            request.url, 'vendor_passthru?method={}'.format(method)
+            request.url, f'vendor_passthru?method={method}'
         )
 
         call = getattr(session, verb.lower())
@@ -1436,7 +1445,7 @@ class Node(_common.ListMixin, resource.Resource):
             )
 
         else:
-            return super(Node, self).patch(
+            return super().patch(
                 session, patch=patch, retry_on_conflict=retry_on_conflict
             )
 
