@@ -15,6 +15,7 @@ from openstack.block_storage.v2 import backup as _backup
 from openstack.block_storage.v2 import capabilities as _capabilities
 from openstack.block_storage.v2 import extension as _extension
 from openstack.block_storage.v2 import limits as _limits
+from openstack.block_storage.v2 import quota_class_set as _quota_class_set
 from openstack.block_storage.v2 import quota_set as _quota_set
 from openstack.block_storage.v2 import snapshot as _snapshot
 from openstack.block_storage.v2 import stats as _stats
@@ -361,7 +362,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v2.volume.Volume` instance.
-        :param str new_type: The new volume type that volume is changed with.
+        :param new_type: The new volume type that volume is changed with.
+            The value can be either the ID of the volume type or a
+            :class:`~openstack.block_storage.v2.type.Type` instance.
         :param str migration_policy: Specify if the volume should be migrated
             when it is re-typed. Possible values are on-demand or never.
             Default: never.
@@ -369,7 +372,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         volume = self._get_resource(_volume.Volume, volume)
-        volume.retype(self, new_type, migration_policy)
+        type_id = resource.Resource._get_id(new_type)
+        volume.retype(self, type_id, migration_policy)
 
     def set_volume_bootable_status(self, volume, bootable):
         """Set bootable status of the volume.
@@ -384,8 +388,38 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.set_bootable_status(self, bootable)
 
+    def set_volume_image_metadata(self, volume, **metadata):
+        """Update image metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume`.
+        :param kwargs metadata: Key/value pairs to be updated in the volume's
+            image metadata. No other metadata is modified by this call.
+
+        :returns: None
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        return volume.set_image_metadata(self, metadata=metadata)
+
+    def delete_volume_image_metadata(self, volume, keys=None):
+        """Delete metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume`.
+        :param list keys: The keys to delete. If left empty complete
+            metadata will be removed.
+
+        :returns: None
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        if keys is not None:
+            for key in keys:
+                volume.delete_image_metadata_item(self, key)
+        else:
+            volume.delete_image_metadata(self)
+
     def reset_volume_status(
-        self, volume, status, attach_status, migration_status
+        self, volume, status=None, attach_status=None, migration_status=None
     ):
         """Reset volume statuses.
 
@@ -618,15 +652,15 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param project: A project to get limits for. The value can be either
             the ID of a project or an
             :class:`~openstack.identity.v2.project.Project` instance.
-        :returns: A Limit object, including both
+        :returns: A Limits object, including both
             :class:`~openstack.block_storage.v2.limits.AbsoluteLimit` and
             :class:`~openstack.block_storage.v2.limits.RateLimit`
-        :rtype: :class:`~openstack.block_storage.v2.limits.Limit`
+        :rtype: :class:`~openstack.block_storage.v2.limits.Limits`
         """
         params = {}
         if project:
             params['project_id'] = resource.Resource._get_id(project)
-        return self._get(_limits.Limit, requires_id=False, **params)
+        return self._get(_limits.Limits, requires_id=False, **params)
 
     # ====== CAPABILITIES ======
     def get_capabilities(self, host):
@@ -641,6 +675,43 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._get(_capabilities.Capabilities, host)
 
+    # ====== QUOTA CLASS SETS ======
+    def get_quota_class_set(self, quota_class_set='default'):
+        """Get a single quota class set
+
+        Only one quota class is permitted, ``default``.
+
+        :param quota_class_set: The value can be the ID of a quota class set
+            (only ``default`` is supported) or a
+            :class:`~openstack.block_storage.v2.quota_class_set.QuotaClassSet`
+            instance.
+
+        :returns: One
+            :class:`~openstack.block_storage.v2.quota_class_set.QuotaClassSet`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_quota_class_set.QuotaClassSet, quota_class_set)
+
+    def update_quota_class_set(self, quota_class_set, **attrs):
+        """Update a QuotaClassSet.
+
+        Only one quota class is permitted, ``default``.
+
+        :param quota_class_set: Either the ID of a quota class set (only
+            ``default`` is supported) or a
+            :class:`~openstack.block_storage.v2.quota_class_set.QuotaClassSet`
+            instance.
+        :param attrs: The attributes to update on the QuotaClassSet represented
+            by ``quota_class_set``.
+
+        :returns: The updated QuotaSet
+        :rtype: :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
+        """
+        return self._update(
+            _quota_class_set.QuotaClassSet, quota_class_set, **attrs
+        )
+
     # ====== QUOTA SETS ======
     def get_quota_set(self, project, usage=False, **query):
         """Show QuotaSet information for the project
