@@ -23,6 +23,7 @@ from openstack.block_storage.v3 import group as _group
 from openstack.block_storage.v3 import group_snapshot as _group_snapshot
 from openstack.block_storage.v3 import group_type as _group_type
 from openstack.block_storage.v3 import limits as _limits
+from openstack.block_storage.v3 import quota_class_set as _quota_class_set
 from openstack.block_storage.v3 import quota_set as _quota_set
 from openstack.block_storage.v3 import resource_filter as _resource_filter
 from openstack.block_storage.v3 import service as _service
@@ -47,7 +48,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         "group": _group.Group,
         "group_snapshot": _group_snapshot.GroupSnapshot,
         "group_type": _group_type.GroupType,
-        "limits": _limits.Limit,
+        "limits": _limits.Limits,
         "quota_set": _quota_set.QuotaSet,
         "resource_filter": _resource_filter.ResourceFilter,
         "snapshot": _snapshot.Snapshot,
@@ -258,6 +259,30 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
         snapshot.set_status(self, status, progress)
 
+    def manage_snapshot(self, **attrs):
+        """Creates a snapshot by using existing storage rather than
+        allocating new storage.
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.block_storage.v3.snapshot.Snapshot`,
+            comprised of the properties on the Snapshot class.
+
+        :returns: The results of snapshot creation
+        :rtype: :class:`~openstack.block_storage.v3.snapshot.Snapshot`
+        """
+        return _snapshot.Snapshot.manage(self, **attrs)
+
+    def unmanage_snapshot(self, snapshot):
+        """Unmanage a snapshot from block storage provisioning.
+
+        :param snapshot: Either the ID of a snapshot or a
+            :class:`~openstack.block_storage.v3.snapshot.Snapshot`.
+
+        :returns: None
+        """
+        snapshot_obj = self._get_resource(_snapshot.Snapshot, snapshot)
+        snapshot_obj.unmanage(self)
+
     # ====== TYPES ======
     def get_type(self, type):
         """Get a single type
@@ -469,7 +494,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             volume_type = self._get_resource(_type.Type, volume_type)
             encryption = self._get(
                 _type.TypeEncryption,
-                volume_type=volume_type.id,
+                volume_type_id=volume_type.id,
                 requires_id=False,
             )
 
@@ -502,7 +527,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             volume_type = self._get_resource(_type.Type, volume_type)
             encryption = self._get(
                 _type.TypeEncryption,
-                volume_type=volume_type.id,
+                volume_type_id=volume_type.id,
                 requires_id=False,
             )
 
@@ -723,7 +748,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
-        :param str new_type: The new volume type that volume is changed with.
+        :param new_type: The new volume type that volume is changed with.
+            The value can be either the ID of the volume type or a
+            :class:`~openstack.block_storage.v3.type.Type` instance.
         :param str migration_policy: Specify if the volume should be migrated
             when it is re-typed. Possible values are on-demand or never.
             Default: never.
@@ -731,7 +758,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         volume = self._get_resource(_volume.Volume, volume)
-        volume.retype(self, new_type, migration_policy)
+        type_id = resource.Resource._get_id(new_type)
+        volume.retype(self, type_id, migration_policy)
 
     def set_volume_bootable_status(self, volume, bootable):
         """Set bootable status of the volume.
@@ -746,8 +774,38 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.set_bootable_status(self, bootable)
 
+    def set_volume_image_metadata(self, volume, **metadata):
+        """Update image metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v3.volume.Volume`.
+        :param kwargs metadata: Key/value pairs to be updated in the volume's
+            image metadata. No other metadata is modified by this call.
+
+        :returns: None
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        return volume.set_image_metadata(self, metadata=metadata)
+
+    def delete_volume_image_metadata(self, volume, keys=None):
+        """Delete metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v3.volume.Volume`.
+        :param list keys: The keys to delete. If left empty complete
+            metadata will be removed.
+
+        :returns: None
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        if keys is not None:
+            for key in keys:
+                volume.delete_image_metadata_item(self, key)
+        else:
+            volume.delete_image_metadata(self)
+
     def reset_volume_status(
-        self, volume, status, attach_status, migration_status
+        self, volume, status=None, attach_status=None, migration_status=None
     ):
         """Reset volume statuses.
 
@@ -809,6 +867,18 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.detach(self, attachment, force, connector)
 
+    def manage_volume(self, **attrs):
+        """Creates a volume by using existing storage rather than
+            allocating new storage.
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.block_storage.v3.volume.Volume`,
+            comprised of the properties on the Volume class.
+        :returns: The results of volume creation
+        :rtype: :class:`~openstack.block_storage.v3.volume.Volume`
+        """
+        return _volume.Volume.manage(self, **attrs)
+
     def unmanage_volume(self, volume):
         """Removes a volume from Block Storage management without removing the
             back-end storage object that is associated with it.
@@ -816,7 +886,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
 
-        :returns: None"""
+        :returns: None
+        """
         volume = self._get_resource(_volume.Volume, volume)
         volume.unmanage(self)
 
@@ -948,9 +1019,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
         :param dict connector: The connector object.
 
-        :returns: None"""
+        :returns: Dictionary containing the modified connector object"""
         volume = self._get_resource(_volume.Volume, volume)
-        volume.init_attachment(self, connector)
+        return volume.init_attachment(self, connector)
 
     def terminate_volume_attachment(self, volume, connector):
         """Update volume status to 'in-use'.
@@ -1212,15 +1283,15 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param project: A project to get limits for. The value can be either
             the ID of a project or an
             :class:`~openstack.identity.v3.project.Project` instance.
-        :returns: A Limit object, including both
+        :returns: A Limits object, including both
             :class:`~openstack.block_storage.v3.limits.AbsoluteLimit` and
             :class:`~openstack.block_storage.v3.limits.RateLimit`
-        :rtype: :class:`~openstack.block_storage.v3.limits.Limit`
+        :rtype: :class:`~openstack.block_storage.v3.limits.Limits`
         """
         params = {}
         if project:
             params['project_id'] = resource.Resource._get_id(project)
-        return self._get(_limits.Limit, requires_id=False, **params)
+        return self._get(_limits.Limits, requires_id=False, **params)
 
     # ====== CAPABILITIES ======
     def get_capabilities(self, host):
@@ -1631,7 +1702,44 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         group_type = self._get_resource(_group_type.GroupType, group_type)
         return group_type.delete_group_specs_property(self, prop)
 
+    # ====== QUOTA CLASS SETS ======
+
+    def get_quota_class_set(self, quota_class_set='default'):
+        """Get a single quota class set
+
+        Only one quota class is permitted, ``default``.
+
+        :param quota_class_set: The value can be the ID of a quota class set
+            (only ``default`` is supported) or a
+            :class:`~openstack.block_storage.v3.quota_class_set.QuotaClassSet`
+            instance.
+
+        :returns: One
+            :class:`~openstack.block_storage.v3.quota_class_set.QuotaClassSet`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_quota_class_set.QuotaClassSet, quota_class_set)
+
+    def update_quota_class_set(self, quota_class_set, **attrs):
+        """Update a QuotaClassSet.
+
+        Only one quota class is permitted, ``default``.
+
+        :param quota_class_set: Either the ID of a quota class set (only
+            ``default`` is supported) or a
+        :param attrs: The attributes to update on the QuotaClassSet represented
+            by ``quota_class_set``.
+
+        :returns: The updated QuotaSet
+        :rtype: :class:`~openstack.block_storage.v3.quota_set.QuotaSet`
+        """
+        return self._update(
+            _quota_class_set.QuotaClassSet, quota_class_set, **attrs
+        )
+
     # ====== QUOTA SETS ======
+
     def get_quota_set(self, project, usage=False, **query):
         """Show QuotaSet information for the project
 
@@ -1710,17 +1818,25 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         name_or_id: str,
         ignore_missing: ty.Literal[True] = True,
         **query,
-    ) -> ty.Optional[_service.Service]:
-        ...
+    ) -> ty.Optional[_service.Service]: ...
 
     @ty.overload
     def find_service(
         self,
         name_or_id: str,
-        ignore_missing: ty.Literal[False] = True,
+        ignore_missing: ty.Literal[False],
         **query,
-    ) -> _service.Service:
-        ...
+    ) -> _service.Service: ...
+
+    # excuse the duplication here: it's mypy's fault
+    # https://github.com/python/mypy/issues/14764
+    @ty.overload
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: bool,
+        **query,
+    ) -> ty.Optional[_service.Service]: ...
 
     def find_service(
         self,
