@@ -17,6 +17,7 @@ from openstack.orchestration.v1 import software_config as _sc
 from openstack.orchestration.v1 import software_deployment as _sd
 from openstack.orchestration.v1 import stack as _stack
 from openstack.orchestration.v1 import stack_environment as _stack_environment
+from openstack.orchestration.v1 import stack_event as _stack_event
 from openstack.orchestration.v1 import stack_files as _stack_files
 from openstack.orchestration.v1 import stack_template as _stack_template
 from openstack.orchestration.v1 import template as _template
@@ -57,7 +58,7 @@ class Proxy(proxy.Proxy):
             # (/stacks/name/id/everything_else), so if on third position we
             # have not a known part - discard it, not to brake further logic
             del url_parts[2]
-        return super(Proxy, self)._extract_name_consume_url_parts(url_parts)
+        return super()._extract_name_consume_url_parts(url_parts)
 
     def read_env_and_templates(
         self,
@@ -240,6 +241,26 @@ class Proxy(proxy.Proxy):
             obj = self._find(_stack.Stack, stack, ignore_missing=False)
         return obj.export(self)
 
+    def suspend_stack(self, stack):
+        """Suspend a stack status
+
+        :param stack: The value can be either the ID of a stack or an instance
+            of :class:`~openstack.orchestration.v1.stack.Stack`.
+        :returns: ``None``
+        """
+        res = self._get_resource(_stack.Stack, stack)
+        res.suspend(self)
+
+    def resume_stack(self, stack):
+        """Resume a stack status
+
+        :param stack: The value can be either the ID of a stack or an instance
+            of :class:`~openstack.orchestration.v1.stack.Stack`.
+        :returns: ``None``
+        """
+        res = self._get_resource(_stack.Stack, stack)
+        res.resume(self)
+
     def get_stack_template(self, stack):
         """Get template used by a stack
 
@@ -588,3 +609,38 @@ class Proxy(proxy.Proxy):
 
         for stack in stacks:
             self.wait_for_delete(stack)
+
+    def stack_events(self, stack, resource_name=None, **attr):
+        """Get a stack events
+
+        :param stack: The value can be the ID of a stack or an instance of
+            :class:`~openstack.orchestration.v1.stack.Stack`
+        :param resource_name: The name of resource. If the resource_name is not None,
+            the base_path changes.
+
+        :returns: A generator of stack_events objects
+        :rtype: :class:`~openstack.orchestration.v1.stack_event.StackEvent`
+        """
+
+        if isinstance(stack, _stack.Stack):
+            obj = stack
+        else:
+            obj = self._get(_stack.Stack, stack)
+
+        if resource_name:
+            base_path = '/stacks/%(stack_name)s/%(stack_id)s/resources/%(resource_name)s/events'
+            return self._list(
+                _stack_event.StackEvent,
+                stack_name=obj.name,
+                stack_id=obj.id,
+                resource_name=resource_name,
+                base_path=base_path,
+                **attr
+            )
+
+        return self._list(
+            _stack_event.StackEvent,
+            stack_name=obj.name,
+            stack_id=obj.id,
+            **attr
+        )
