@@ -17,7 +17,7 @@ from openstack.tests.functional import base
 
 class TestFlavor(base.BaseFunctionalTest):
     def setUp(self):
-        super(TestFlavor, self).setUp()
+        super().setUp()
         self.new_item_name = self.getUniqueString('flavor')
         self.one_flavor = list(self.conn.compute.flavors())[0]
 
