@@ -20,8 +20,10 @@ from openstack.block_storage.v3 import group
 from openstack.block_storage.v3 import group_snapshot
 from openstack.block_storage.v3 import group_type
 from openstack.block_storage.v3 import limits
+from openstack.block_storage.v3 import quota_class_set
 from openstack.block_storage.v3 import quota_set
 from openstack.block_storage.v3 import resource_filter
+from openstack.block_storage.v3 import service
 from openstack.block_storage.v3 import snapshot
 from openstack.block_storage.v3 import stats
 from openstack.block_storage.v3 import type
@@ -32,7 +34,7 @@ from openstack.tests.unit import test_proxy_base
 
 class TestVolumeProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestVolumeProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -136,7 +138,7 @@ class TestLimit(TestVolumeProxy):
     def test_limits_get(self):
         self.verify_get(
             self.proxy.get_limits,
-            limits.Limit,
+            limits.Limits,
             method_args=[],
             expected_kwargs={'requires_id': False},
         )
@@ -316,6 +318,53 @@ class TestGroupType(TestVolumeProxy):
         )
 
 
+class TestService(TestVolumeProxy):
+    def test_services(self):
+        self.verify_list(self.proxy.services, service.Service)
+
+    def test_enable_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.enable',
+            self.proxy.enable_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_disable_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.disable',
+            self.proxy.disable_service,
+            method_args=["value"],
+            expected_kwargs={"reason": None},
+            expected_args=[self.proxy],
+        )
+
+    def test_thaw_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.thaw',
+            self.proxy.thaw_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_freeze_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.freeze',
+            self.proxy.freeze_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_failover_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.failover',
+            self.proxy.failover_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+            expected_kwargs={"backend_id": None, "cluster": None},
+        )
+
+
 class TestExtension(TestVolumeProxy):
     def test_extensions(self):
         self.verify_list(self.proxy.extensions, extension.Extension)
@@ -362,6 +411,32 @@ class TestVolumeActions(TestVolumeProxy):
             expected_args=[self.proxy, '1', '2', '3'],
         )
 
+    def test_set_volume_image_metadata(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.set_image_metadata",
+            self.proxy.set_volume_image_metadata,
+            method_args=["value"],
+            method_kwargs={'foo': 'bar'},
+            expected_args=[self.proxy],
+            expected_kwargs={'metadata': {'foo': 'bar'}},
+        )
+
+    def test_delete_volume_image_metadata(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.delete_image_metadata",
+            self.proxy.delete_volume_image_metadata,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_delete_volume_image_metadata__with_keys(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.delete_image_metadata_item",
+            self.proxy.delete_volume_image_metadata,
+            method_args=["value", ['foo']],
+            expected_args=[self.proxy, 'foo'],
+        )
+
     def test_volume_revert_to_snapshot(self):
         self._verify(
             "openstack.block_storage.v3.volume.Volume.revert_to_snapshot",
@@ -737,6 +812,24 @@ class TestSnapshot(TestVolumeProxy):
             expected_args=[self.proxy, "key"],
         )
 
+    def test_manage_snapshot(self):
+        kwargs = {
+            "volume_id": "fake_id",
+            "remote_source": "fake_volume",
+            "snapshot_name": "fake_snap",
+            "description": "test_snap",
+            "property": {"k": "v"},
+        }
+        self._verify(
+            "openstack.block_storage.v3.snapshot.Snapshot.manage",
+            self.proxy.manage_snapshot,
+            method_kwargs=kwargs,
+            method_result=snapshot.Snapshot(id="fake_id"),
+            expected_args=[self.proxy],
+            expected_kwargs=kwargs,
+            expected_result=snapshot.Snapshot(id="fake_id"),
+        )
+
 
 class TestType(TestVolumeProxy):
     def test_type_get(self):
@@ -825,11 +918,27 @@ class TestType(TestVolumeProxy):
         )
 
     def test_type_encryption_update(self):
+        # Verify that the get call was made with correct kwargs
+        self.verify_get(
+            self.proxy.get_type_encryption,
+            type.TypeEncryption,
+            method_args=['value'],
+            expected_args=[],
+            expected_kwargs={'volume_type_id': 'value', 'requires_id': False},
+        )
         self.verify_update(
             self.proxy.update_type_encryption, type.TypeEncryption
         )
 
     def test_type_encryption_delete(self):
+        # Verify that the get call was made with correct kwargs
+        self.verify_get(
+            self.proxy.get_type_encryption,
+            type.TypeEncryption,
+            method_args=['value'],
+            expected_args=[],
+            expected_kwargs={'volume_type_id': 'value', 'requires_id': False},
+        )
         self.verify_delete(
             self.proxy.delete_type_encryption, type.TypeEncryption, False
         )
@@ -840,8 +949,22 @@ class TestType(TestVolumeProxy):
         )
 
 
-class TestQuota(TestVolumeProxy):
-    def test_get(self):
+class TestQuotaClassSet(TestVolumeProxy):
+    def test_quota_class_set_get(self):
+        self.verify_get(
+            self.proxy.get_quota_class_set, quota_class_set.QuotaClassSet
+        )
+
+    def test_quota_class_set_update(self):
+        self.verify_update(
+            self.proxy.update_quota_class_set,
+            quota_class_set.QuotaClassSet,
+            False,
+        )
+
+
+class TestQuotaSet(TestVolumeProxy):
+    def test_quota_set_get(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
@@ -856,7 +979,7 @@ class TestQuota(TestVolumeProxy):
             expected_result=quota_set.QuotaSet(),
         )
 
-    def test_get_query(self):
+    def test_quota_set_get_query(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
@@ -871,7 +994,7 @@ class TestQuota(TestVolumeProxy):
             },
         )
 
-    def test_get_defaults(self):
+    def test_quota_set_get_defaults(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set_defaults,
@@ -884,7 +1007,7 @@ class TestQuota(TestVolumeProxy):
             },
         )
 
-    def test_reset(self):
+    def test_quota_set_reset(self):
         self._verify(
             'openstack.resource.Resource.delete',
             self.proxy.revert_quota_set,
@@ -895,7 +1018,7 @@ class TestQuota(TestVolumeProxy):
         )
 
     @mock.patch('openstack.proxy.Proxy._get_resource', autospec=True)
-    def test_update(self, gr_mock):
+    def test_quota_set_update(self, gr_mock):
         gr_mock.return_value = resource.Resource()
         gr_mock.commit = mock.Mock()
         self._verify(
