@@ -10,31 +10,32 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack.identity.v3 import role
 from openstack.tests.unit import base
 
-from openstack.identity.v3 import role
 
 IDENTIFIER = 'IDENTIFIER'
 EXAMPLE = {
     'id': IDENTIFIER,
     'links': {'self': 'http://example.com/user1'},
     'name': '2',
+    'description': 'test description for role',
+    'domain_id': 'default',
 }
 
 
 class TestRole(base.TestCase):
-
     def test_basic(self):
         sot = role.Role()
         self.assertEqual('role', sot.resource_key)
         self.assertEqual('roles', sot.resources_key)
         self.assertEqual('/roles', sot.base_path)
-        self.assertEqual('identity', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
+        self.assertEqual('PATCH', sot.commit_method)
 
         self.assertDictEqual(
             {
@@ -43,10 +44,13 @@ class TestRole(base.TestCase):
                 'limit': 'limit',
                 'marker': 'marker',
             },
-            sot._query_mapping._mapping)
+            sot._query_mapping._mapping,
+        )
 
     def test_make_it(self):
         sot = role.Role(**EXAMPLE)
         self.assertEqual(EXAMPLE['id'], sot.id)
         self.assertEqual(EXAMPLE['links'], sot.links)
         self.assertEqual(EXAMPLE['name'], sot.name)
+        self.assertEqual(EXAMPLE['description'], sot.description)
+        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
